<!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
		XSLT-Stylesheet responsible for transforming gbXML to RDF/OWL
		Author: Mario J. Kofler
		Date: June 2010
		Version: 0.1
		Description: 
		Transfers the gbXML Schema to RDF/OWL. Takes Approach independent
		of actual instatiations of gbXML elements.
	 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-->
<!DOCTYPE rdf:RDF [
    <!ENTITY owl "http://www.w3.org/2002/07/owl#" >
    <!ENTITY xsd "http://www.w3.org/2001/XMLSchema#" >
    <!ENTITY owl2xml "http://www.w3.org/2006/12/owl2-xml#" >
    <!ENTITY rdfs "http://www.w3.org/2000/01/rdf-schema#" >
    <!ENTITY rdf "http://www.w3.org/1999/02/22-rdf-syntax-ns#" >
    <!ENTITY gbOnt "http://www.auto.tuwien.ac.at/downloads/thinkhome/ontology/BuildingOntology.owl#" >
    <!ENTITY gbdt "http://www.gbxml.org/schema/0-37/GreenBuildingXML.xsd#" >
]>

	<xsl:stylesheet version="2.0" 
					xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
					xmlns:xsd="http://www.w3.org/2001/XMLSchema"
					xmlns:owl="http://www.w3.org/2002/07/owl#"
					xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
					xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"
					xmlns:fkt="http://www.auto.tuwien.ac.at/mkofler/functions"
					xmlns:gbdt="http://www.gbxml.org/schema/0-37/GreenBuildingXML.xsd#"
					xmlns:BuildingOntology="http://www.auto.tuwien.ac.at/downloads/thinkhome/ontology/gbBuilding.owl#"
					exclude-result-prefixes="fkt gbdt"
					> 	
	
	<xsl:template match="xsd:simpleType" mode="datatype">
		<xsl:element name="rdfs:Datatype">
			<xsl:attribute name="rdf:about">
				<xsl:value-of select="concat('#',string(@name))"/>
			</xsl:attribute>
			<xsl:element name="owl:equivalentClass">
				<xsl:element name="rdfs:Datatype">
					<xsl:element name="owl:oneOf">
						<xsl:value-of 	disable-output-escaping="yes"
										select="fkt:simpleList(normalize-space(string-join(xsd:restriction/xsd:enumeration/@value,' ')),'&amp;xsd;string')"/>
					</xsl:element>
				</xsl:element>
			</xsl:element>
		</xsl:element>
	</xsl:template>
	
	<!-- Templaterule for gbXML -->
	<xsl:template match="xsd:schema" mode="class">
	
	<!-- declaration of globally needed datatypes. -->
	<xsl:apply-templates select="./xsd:simpleType" mode="datatype"/>
	
	<!-- declaration of globally needed datatype/object properties -->
	<!-- the hasNativeValue property is needed by many of the elements who just have a value and a unit, for example temperature -->
	<xsl:value-of 	disable-output-escaping="yes"
				 	select="fkt:createProperty('Datatype',concat('#','hasNativeValue'),'&amp;owl;FunctionalProperty','','','')"/>

	<!-- create DatatypeProperties -->
	<xsl:call-template name="createDatatypeProperties"/>

	<!-- create ObjectProperties -->
	<xsl:call-template name="createObjectProperties"/>
	
	<!--  create classes -->			 	
	<xsl:for-each select="xsd:element">
		<xsl:choose>
			<xsl:when test="count(child::*)=0">
				<xsl:choose>
					<xsl:when test="@type">
						<xsl:variable name="elem_exists" select="document('../../../XSD/GreenBuildingXML.xsd')//xsd:attribute[@name=lower-case(current-grouping-key())]"/>
						<xsl:choose>
							<!--  workaround for the odd case that element Version and attribute version exists -->
							<xsl:when test="not(@name='Version')">
								<xsl:value-of 	disable-output-escaping="yes"
								  	  			select="fkt:createProperty('Datatype',concat('#',fkt:createDPName(@name)),'','',fkt:testOfType(@type),'')"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of 	disable-output-escaping="yes"
								  	  			select="fkt:createProperty('Datatype',concat('#',fkt:createDPName(@name)),'','','','')"/>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="createClass">
							<xsl:with-param  name="pName" select="./@name"/>
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="count(child::*)=1">
				<xsl:choose>
					<xsl:when test="child::*/local-name()='annotation'">
						<xsl:value-of disable-output-escaping="yes"
									  select="fkt:createProperty('Datatype',concat('#',fkt:createDPName(@name)),'','',fkt:testOfType(@type),fkt:createAnnotation('comment',xsd:annotation/xsd:documentation))"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="createClass">
							<xsl:with-param  name="pName" select="./@name"/>
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="xsd:simpleType/xsd:restriction">
				<xsl:value-of 	disable-output-escaping="yes"
							  	  select="fkt:createProperty('Datatype',concat('#',fkt:createDPName(@name)),'','',fkt:testOfType(xsd:simpleType/xsd:restriction/@base),'')"/>
			</xsl:when>
	   		<xsl:otherwise>
				<xsl:call-template name="createClass">
					<xsl:with-param  name="pName" select="./@name"/>
				</xsl:call-template>
			</xsl:otherwise>
	   	</xsl:choose>
	</xsl:for-each>
	
	<!-- make all classes disjoint to each other -->
    <xsl:element name="rdf:Description">
        <xsl:element name="rdf:type">
        	<xsl:attribute name="rdf:resource">
        		<xsl:value-of 	disable-output-escaping="yes"
        						select="'&owl;AllDisjointClasses'"/>
        	</xsl:attribute>
        </xsl:element>
       	<xsl:element name="owl:members"> 
       		<xsl:attribute name="rdf:parseType">
       			<xsl:value-of select="'Collection'"/>
       		</xsl:attribute>
			<xsl:for-each select="xsd:element">
				<xsl:if test="not(@name = 'UtilityRate' or
	 					  		  @name = 'aecXML' or
	 					  		  @name = 'GeneralGeometry')">
					<xsl:choose>
						<xsl:when test="count(child::*)=0">
							<xsl:choose>
								<xsl:when test="@type">
									<!-- do nothing -->
								</xsl:when>
								<xsl:otherwise>
									<xsl:element name="rdf:Description">
										<xsl:attribute name="rdf:about">
											<!-- construct qualified name: the # creates a connection to the xml:base namespace -->
											<xsl:value-of select="concat('#',./@name)"/> 
										</xsl:attribute>
									</xsl:element>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:when>
						<xsl:when test="count(child::*)=1">
							<xsl:choose>
								<xsl:when test="child::*/local-name()='annotation'">
									<!-- do nothing -->
								</xsl:when>
								<xsl:otherwise>
									<xsl:element name="rdf:Description">
										<xsl:attribute name="rdf:about">
											<!-- construct qualified name: the # creates a connection to the xml:base namespace -->
											<xsl:value-of select="concat('#',./@name)"/>
										</xsl:attribute>
									</xsl:element>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:when>
						<xsl:when test="xsd:simpleType/xsd:restriction">
							<!-- do nothing -->
						</xsl:when>
				   		<xsl:otherwise>
							<xsl:element name="rdf:Description">
								<xsl:attribute name="rdf:about">
									<!-- construct qualified name: the # creates a connection to the xml:base namespace -->
									<xsl:value-of select="concat('#',./@name)"/>
								</xsl:attribute>
							</xsl:element>
						</xsl:otherwise>
				   	</xsl:choose>
				</xsl:if>
			</xsl:for-each>	
		</xsl:element>
	</xsl:element>
	</xsl:template>

	<!-- simple datatype Property helper template to realize the comparison with the element name -->
	<xsl:template name="helperSingleTyped">
		<xsl:param name="pRefName"/>
		<xsl:param name="pMinCardinality"/>
		<xsl:param name="pMaxCardinality"/>
	 	<xsl:for-each select="/xsd:schema/xsd:element">
	 		<xsl:if test="not(@name = 'UtilityRate' or
	  				  		  @name = 'aecXML' or
	  				  		  @name = 'GeneralGeometry')">
			<xsl:choose>
				<xsl:when test="@name=$pRefName">
					<xsl:choose>
						<xsl:when test="count(child::*)=0">
							<xsl:value-of 	disable-output-escaping="yes"
							select="fkt:existentialDPRestriction(fkt:createDPName($pRefName),fkt:testOfType(@type),$pMinCardinality,$pMaxCardinality)"/>
						</xsl:when>
						<xsl:when test="count(child::*)=1">
							<xsl:choose>
								<xsl:when test="child::*/local-name()='annotation'">
									<xsl:choose>
										<xsl:when test="@type">
											<xsl:value-of 	disable-output-escaping="yes"
																	select="fkt:existentialDPRestriction(fkt:createDPName($pRefName),fkt:testOfType(@type),$pMinCardinality,$pMaxCardinality)"/>	
										</xsl:when>
										<xsl:otherwise>
											<xsl:value-of 	disable-output-escaping="yes"
															select="fkt:existentialDPRestriction(fkt:createDPName($pRefName),fkt:testOfType('xsd:string'),$pMinCardinality,$pMaxCardinality)"/>
										</xsl:otherwise>
									</xsl:choose>
								</xsl:when>
								<xsl:otherwise>
									<xsl:choose>
										<xsl:when test="@type">
											<xsl:value-of 	disable-output-escaping="yes"
															select="fkt:existentialDPRestriction(fkt:createDPName($pRefName),fkt:testOfType(@type),$pMinCardinality,$pMaxCardinality)"/>
										</xsl:when>
										<xsl:otherwise>														
										 	<!-- construct qualified name for filler class ($pRefName): the # creates a connection to the xml:base namespace -->
										 	<xsl:value-of 	disable-output-escaping="yes"
															select="fkt:existentialOPRestriction(concat('contains',fkt:capitalize-first($pRefName)),concat('#',$pRefName),$pMinCardinality,$pMaxCardinality)"/>
										</xsl:otherwise>
									</xsl:choose>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:when>
						<!-- if the element has a simpleType restriction, then take the value of the "base attribute" as datatype, as this attribute is the datatype
							 of the base element-->
						<xsl:when test="./xsd:simpleType/xsd:restriction">
							<xsl:variable name="dpType" select="if 	 (./xsd:simpleType/xsd:restriction/@base)
																then (fkt:testOfType(./xsd:simpleType/xsd:restriction[1]/@base))
																else (fkt:testOfType('xsd:string'))"/>
							<xsl:value-of 	disable-output-escaping="yes"
											select="fkt:existentialDPRestriction(fkt:createDPName($pRefName),$dpType,$pMinCardinality,$pMaxCardinality)"/>
						</xsl:when>
						<xsl:otherwise>
								<!-- construct qualified name for filler class ($pRefName): the # creates a connection to the xml:base namespace -->
 								<xsl:value-of 	disable-output-escaping="yes"
											select="fkt:existentialOPRestriction(concat('contains',fkt:capitalize-first($pRefName)),concat('#',$pRefName),$pMinCardinality,$pMaxCardinality)"/> 
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
			</xsl:choose>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
	
	
	<!-- template for linking and restricting class properties to class (hasValue,object properties, datatype properties)
		 in this case: attributes => datatype properties -->
	<xsl:template match="xsd:complexType | xsd:complexType/xsd:simpleContent/xsd:extension" mode="class">
					<xsl:if test="(count(child::*) &gt; 1) or (count(child::*)=1 and @base)">	
						<xsl:value-of 	disable-output-escaping="yes"
										select="'&lt;owl:intersectionOf rdf:parseType=&quot;Collection&quot;&gt;'"/>
				  	</xsl:if>
				  	
					<xsl:if test="@base">
						<xsl:value-of 	disable-output-escaping="yes"
										select="fkt:existentialDPRestriction('hasNativeValue',fkt:testOfType(@base),'1','1')"/> 
					</xsl:if>
					<xsl:if test="(xsd:choice/xsd:element)">
						<xsl:element name="owl:Class">
							<xsl:element name="owl:unionOf">
								<xsl:attribute name="rdf:parseType">
			   						<xsl:value-of select="string('Collection')"/>
				  				</xsl:attribute>
				  				
				  				<xsl:apply-templates select="./xsd:choice" mode="class"/>
								
							</xsl:element>
						</xsl:element>
					</xsl:if>
			  		<xsl:for-each select="xsd:attribute">
			  			<xsl:choose>
			  				<xsl:when test="@use='required'">
					  			<xsl:choose>
				  					<xsl:when test="@name and @type ">
				  						<!-- create existential restriction for this DP with type and everything. -->
				  						<xsl:choose>
				  							<!-- test if it is a native xsd: datatype. if yes, use it as type, else do not use it and use xsd:string instead -->	
				  							<xsl:when test="substring(@type,1,4)='xsd:'">
					  							<xsl:value-of 	disable-output-escaping="yes"
																select="fkt:existentialDPRestriction(fkt:createDPName(@name),fkt:testOfType(@type),'1','1')"/>
											</xsl:when>
											<xsl:otherwise>
												<xsl:value-of 	disable-output-escaping="yes"
																select="fkt:existentialDPRestriction(fkt:createDPName(@name),fkt:testOfType('xsd:string'),'1','1')"/>
											</xsl:otherwise>
										</xsl:choose>
				  					</xsl:when>
				  					<xsl:when test="@name">			
				  						<xsl:value-of 	disable-output-escaping="yes"
														select="fkt:existentialDPRestriction(fkt:createDPName(@name),fkt:testOfType('xsd:string'),'1','1')"/>
				  					</xsl:when>
				  				</xsl:choose>
				  			</xsl:when>
				  			<xsl:otherwise>
				  				<xsl:choose>
				  					<xsl:when test="@name and @type ">
				  						<xsl:choose>
				  							<!-- test if it is a native xsd: datatype. if yes, use it as type, else do not use it and use xsd:string instead -->
				  							<xsl:when test="substring(@type,1,4)='xsd:'">
					  							<xsl:value-of 	disable-output-escaping="yes" 
																select="fkt:existentialDPRestriction(fkt:createDPName(@name),fkt:testOfType(@type),'','1')"/>
											</xsl:when>
											<xsl:otherwise>
												<xsl:value-of 	disable-output-escaping="yes" 
																select="fkt:existentialDPRestriction(fkt:createDPName(@name),fkt:testOfType('xsd:string'),'','1')"/>
											</xsl:otherwise>
										</xsl:choose>
				  					</xsl:when>
				  					<xsl:when test="@name">		
				  						<xsl:value-of 	disable-output-escaping="yes" 
														select="fkt:existentialDPRestriction(fkt:createDPName(@name),fkt:testOfType('xsd:string'),'','1')"/>
				  					</xsl:when>
				  				</xsl:choose>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:for-each>
				<xsl:if test="(count(child::*) &gt; 1) or (count(child::*)=1 and @base)">	
					<xsl:value-of 	disable-output-escaping="yes"
									select="'&lt;/owl:intersectionOf&gt;'"/>
				</xsl:if>
	</xsl:template>
	
	<!-- template for linking and restricting class properties to class (hasValue,object properties, datatype properties)
		 in this case: attributes => datatype properties -->
	<xsl:template match="xsd:complexType/xsd:choice" mode="class">
  		<xsl:for-each select="xsd:element">
  			<xsl:call-template name="helperSingleTyped">
				<xsl:with-param  name="pRefName" select="@ref"/>
				<xsl:with-param name="pMinCardinality" select="if(not(@minOccurs)) 	then '1' else @minOccurs"/>
				<xsl:with-param name="pMaxCardinality" select="if(not(@maxOccurs)) 	then '1' else if (@maxOccurs='unbounded') then '666' else @maxOccurs"/>
			</xsl:call-template>
		</xsl:for-each>
	</xsl:template>	
	
	<!-- template for linking and restricting class properties to class (hasValue,object properties, datatype properties)
		 in this case: attributes => datatype properties -->
	<xsl:template match="xsd:simpleType" mode="class">
		<xsl:for-each-group select="xsd:restriction" group-by="@base">
			<xsl:value-of 	disable-output-escaping="yes"
							select="fkt:existentialDPRestriction(fkt:createDPName(../../@name),fkt:testOfType(current-grouping-key()),'','')"/>
		</xsl:for-each-group>
	</xsl:template>	
	
	<xsl:template name="createClass">
		<xsl:param name="pName"/>
		<xsl:if test="not(@name = 'UtilityRate' or
	  					  @name = 'aecXML' or
	  					  @name = 'GeneralGeometry')">
			<xsl:element name="owl:Class">
	    		<xsl:attribute name="rdf:about">
	   				<xsl:value-of select="concat('#',./@name)"/><!-- anchor in order to associate the namespace xml:base -->
	  			</xsl:attribute>
	   			<xsl:if test="child::*/local-name()='annotation'">
					<xsl:value-of disable-output-escaping="yes"
				  				  select="fkt:createAnnotation('comment',xsd:annotation/xsd:documentation)"/>
				</xsl:if>
				<xsl:element name="rdfs:subClassOf">
				<xsl:choose>
					<xsl:when test="((count(xsd:complexType/xsd:simpleContent/xsd:extension) = 1) and 
									 count(xsd:complexType/xsd:simpleContent/xsd:extension/child::*) = 0) or
									 (count(xsd:complexType/xsd:attribute)=1 and not(xsd:complexType/xsd:choice/xsd:element))">
						<xsl:choose>
						<xsl:when test="(count(xsd:complexType/xsd:simpleContent/xsd:extension) = 1) and 
									 count(xsd:complexType/xsd:simpleContent/xsd:extension/child::*) = 0">
							<xsl:apply-templates select="xsd:complexType/xsd:simpleContent/xsd:extension" mode="class"/>
						</xsl:when>
						<xsl:when test="(xsd:complexType/xsd:attribute)">
							<xsl:apply-templates select="xsd:complexType" mode="class"/>
						</xsl:when>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise>
						<xsl:element name="owl:Class"> 
					  			<xsl:choose>
					  				<xsl:when test="xsd:complexType/xsd:simpleContent/xsd:extension">				
										<xsl:apply-templates select="xsd:complexType/xsd:simpleContent/xsd:extension" mode="class"/>
										<xsl:if test="(xsd:complexType/xsd:attribute) and (xsd:complexType/xsd:attribute/@use='required')">
											<xsl:apply-templates select="xsd:complexType" mode="class"/>
										</xsl:if>
					  				</xsl:when>
					  				<xsl:when test="xsd:complexType/xsd:choice/xsd:element">
						  				<xsl:choose>
						  					<!-- when optional additional attribute to choice/element then apply template: -->
							  				<xsl:when test="xsd:complexType/xsd:attribute">
							  					<xsl:apply-templates select="xsd:complexType" mode="class"/>
							  				</xsl:when>
							  				<xsl:otherwise>
												<xsl:element name="owl:unionOf">
													<xsl:attribute name="rdf:parseType">
							   							<xsl:value-of select="string('Collection')"/>
								  					</xsl:attribute>
													<xsl:apply-templates select="xsd:complexType/xsd:choice" mode="class"/>
												</xsl:element>
											</xsl:otherwise>
										</xsl:choose>
					  				</xsl:when>
					  				<xsl:when test="xsd:complexType/xsd:attribute">
										<xsl:apply-templates select="xsd:complexType" mode="class"/>
							  			
									</xsl:when>
					  			</xsl:choose>

								<xsl:if test="xsd:simpleType">
								
									<xsl:apply-templates select="xsd:simpleType" mode="class"/>
									
								</xsl:if>
						</xsl:element>
					</xsl:otherwise>
					</xsl:choose>
				</xsl:element>
	   		</xsl:element>
	   	</xsl:if>
	</xsl:template>
	
</xsl:stylesheet>