<!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
		XSLT-Stylesheet responsible for transforming gbXML to RDF/OWL
		Author: Mario J. Kofler
		Date: June 2010
		Version: 0.1
		Description: 
		Provides general transfer functions
	 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-->
<xsl:stylesheet version="2.0" 
				xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
				xmlns:gbx="http://www.gbxml.org/schema"
				xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"
				xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
				xmlns:owl="http://www.w3.org/2002/07/owl#"
				xmlns:fkt="http://www.auto.tuwien.ac.at/mkofler/functions"
				xmlns:xsd="http://www.w3.org/2001/XMLSchema"
				xmlns:gbdt="http://www.gbxml.org/schema/0-37/GreenBuildingXML.xsd#"
				exclude-result-prefixes="fkt gbdt"
				>
				
				
<!--XSLT-FUNCTX Helper: 
	returns the substring after the last matching occurrence of regex -->				
<xsl:function name="fkt:substring-after-last-match" >
  <xsl:param name="arg" /> 
  <xsl:param name="regex" /> 
 
  <xsl:sequence select=" 
   replace($arg,concat('^.*',$regex),'')
 "/>
   
</xsl:function>

<!--XSLT-FUNCTX Helper: 
	capitalizes first letter of a string -->
<xsl:function name="fkt:capitalize-first" >
  <xsl:param name="arg" as="xsd:string?"/> 
 
  <xsl:sequence select=" 
   concat(upper-case(substring($arg,1,1)),
             substring($arg,2))
 "/>
</xsl:function>

<!-- function testOfType to test and return the type for the target document
	 param1: type value -->
<xsl:function name="fkt:testOfType">
	<xsl:param name="pType"/>
	<xsl:choose>
	<xsl:when test="substring-after($pType,'xsd:')!=''">
		<!--  ADDITION 07.06.2010: ID and IDREF have to be changed to String because these datatypes are not defined in RDF semantics  -->
		<xsl:choose>
			<xsl:when test="substring-after($pType,'xsd:')='ID' or substring-after($pType,'xsd:')='IDREF'">
				<xsl:value-of select="concat('&amp;xsd;','string')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:choose>
					<xsl:when test="substring-after($pType,'xsd:')='decimal'">
						<xsl:value-of select="concat('&amp;xsd;','double')"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="concat('&amp;xsd;',substring-after($pType,'xsd:'))"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:when>
	<!--  COMMENT: the gbXML sometimes is buggy and for example refers to a type powerPerEnum which NEVER was declared as simple type -->
	<xsl:when test="$pType='powerPerEnum'">
		<xsl:value-of select="concat('&amp;xsd;','string')"/>
	</xsl:when>
	<xsl:otherwise>
		<!-- CHANGED 07.04.: DO NOTHING -->
		<xsl:value-of select="concat('#',$pType)"/>
	</xsl:otherwise>
	</xsl:choose>
</xsl:function>

<!-- function to create general AllDisjoint axiom
	 param1: restriction type (allDisjoints)
	 param2: disjoint classes -->
<xsl:function name="fkt:allDisjoints">
	<xsl:param name="pType"/>
	<xsl:param name="pClass"/>
	<xsl:value-of select="concat('&lt;','rdf:Description','&gt;')"/>
	<xsl:value-of select="concat('&lt;','rdf:type rdf:resource=&quot;',concat('#',$pType),'&quot;/&gt;')"/> <!-- anchor to associate with xml:base namespace -->
	<xsl:value-of select="concat('&lt;','owl:members rdf:parseType=&quot;Collection&quot;','&gt;')"/>
	<xsl:variable name="pClasses" select="tokenize($pClass,'\s')"/>
	<xsl:for-each select="$pClasses">
		<!-- with concat # the qualified name of the resource is created and thus connected to the xml:base namespace -->
		<xsl:value-of select="concat('&lt;','rdf:Description rdf:about=&quot;',concat('#',.),'&quot;/&gt;')"/>
	</xsl:for-each>
	<xsl:value-of select="concat('&lt;','/owl:members','&gt;')"/>
	
	<xsl:value-of select="concat('&lt;/','rdf:Description','&gt;')"/>
	
</xsl:function>

<!-- function to create OWL unionOf construct
	 param1: elements to create unionof-->
<xsl:function name="fkt:unionOf">
	<xsl:param name="pUnionClasses"/>
	
	<xsl:value-of select="concat('&lt;','owl:Class','&gt;')"/>
	<xsl:value-of select="concat('&lt;','owl:UnionOf rdf:parseType=&quot;Collection&quot;','&gt;')"/>
	<xsl:variable name="pUnionClasses" select="tokenize(string-join($pUnionClasses,' '),' ')"/>
	<xsl:for-each select="$pUnionClasses">
		<!-- with concat # the qualified name of the resource is created and thus connected to the xml:base namespace -->
		<xsl:value-of select="concat('&lt;','rdf:Description rdf:about=&quot;',concat('#',.),'&quot;/&gt;')"/>
	</xsl:for-each>
	<xsl:value-of select="concat('&lt;','owl:UnionOf rdf:parseType=&quot;Collection&quot;','&gt;')"/>
	<xsl:value-of select="concat('&lt;','owl:Class','&gt;')"/>
</xsl:function>

<!-- function to create a standardized Datatype property name -->
<xsl:function name="fkt:createDPName">
	<xsl:param name="pDPName"/>
	<xsl:choose>
	<xsl:when test="$pDPName='Value'">
		<xsl:value-of select="concat('has',fkt:capitalize-first($pDPName),'Element')"/>
	</xsl:when>
	<xsl:when test="substring-before($pDPName,'Value')=''">
		<xsl:value-of select="concat('has',fkt:capitalize-first($pDPName),'Value')"/>
	</xsl:when>
	<xsl:otherwise>
		<xsl:value-of select="concat('has',fkt:capitalize-first(substring-before($pDPName,'Value')),'Value')"/>
	</xsl:otherwise>
	</xsl:choose>
</xsl:function>

<!-- function to create owl Annotation
	 param1:  Annotation type (one of the standard rdfs annotation types label/comment/seeAlso/isDefinedBy)
	 param2:  value of the annotation -->
<xsl:function name="fkt:createAnnotation">
	<xsl:param name="pAnnType"/>
	<xsl:param name="pAnnVal"/>
	<xsl:if test="$pAnnVal!=''">
		<xsl:value-of select="concat('&lt;rdfs:',$pAnnType,' rdf:datatype=&quot;&amp;xsd;string&quot; &gt;')"/>
		<xsl:value-of select="$pAnnVal"/>
		<xsl:value-of select="concat('&lt;/rdfs:',$pAnnType,'&gt;')"/>
	</xsl:if>
</xsl:function>

<!--  function to create existential restrictions inside a class definition for intersection of restrictions
	  param1: name of the restriction property
	  param2: name of the filler class/datatype-->
<xsl:function name="fkt:existentialDPRestriction">
	<xsl:param name="pProp"/>
	<xsl:param name="pFiller"/>
	<xsl:param name="pMinValue"/>
	<xsl:param name="pMaxValue"/>
			<xsl:choose>
				<xsl:when test="($pMinValue!='' or $pMaxValue!='') and $pFiller!=''">
					<xsl:choose>
						<xsl:when test="$pMaxValue ='666'">
							<xsl:value-of select="concat('&lt;','owl:Restriction','&gt;')"/>
							<xsl:value-of select="concat('&lt;owl:onProperty rdf:resource=&quot;',concat('#',$pProp),'&quot;/&gt;')"/>
							<xsl:value-of select="concat('&lt;','owl:onDataRange rdf:resource=&quot;',$pFiller,'&quot;/&gt;')"/>
							<xsl:value-of select="concat('&lt;','owl:minQualifiedCardinality rdf:datatype=&quot;&amp;xsd;nonNegativeInteger&quot;&gt;')"/>
							<xsl:value-of select="$pMinValue"/>
							<xsl:value-of select="concat('&lt;','/owl:minQualifiedCardinality&gt;')"/>
							<xsl:value-of select="concat('&lt;/','owl:Restriction','&gt;')"/>
						</xsl:when>
						<xsl:when test="$pMinValue &gt;'0' and $pMaxValue=''">
							<xsl:value-of select="concat('&lt;','owl:Restriction','&gt;')"/>
							<xsl:value-of select="concat('&lt;owl:onProperty rdf:resource=&quot;',concat('#',$pProp),'&quot;/&gt;')"/>
							<xsl:value-of select="concat('&lt;','owl:onDataRange rdf:resource=&quot;',$pFiller,'&quot;/&gt;')"/>
							<xsl:value-of select="concat('&lt;','owl:minQualifiedCardinality rdf:datatype=&quot;&amp;xsd;nonNegativeInteger&quot;&gt;')"/>
							<xsl:value-of select="$pMinValue"/>
							<xsl:value-of select="concat('&lt;','/owl:minQualifiedCardinality&gt;')"/>
							<xsl:value-of select="concat('&lt;/','owl:Restriction','&gt;')"/>
						</xsl:when>
						<!-- min cardinality 0 does not have to be written in order to keep the ontology more readful -->
						<xsl:when test="$pMinValue='' and $pMaxValue!=''">
							<xsl:value-of select="concat('&lt;','owl:Restriction','&gt;')"/>
							<xsl:value-of select="concat('&lt;owl:onProperty rdf:resource=&quot;',concat('#',$pProp),'&quot;/&gt;')"/>
							<xsl:value-of select="concat('&lt;','owl:onDataRange rdf:resource=&quot;',$pFiller,'&quot;/&gt;')"/>
							<xsl:value-of select="concat('&lt;','owl:maxQualifiedCardinality rdf:datatype=&quot;&amp;xsd;nonNegativeInteger&quot;&gt;')"/>
							<xsl:value-of select="string($pMaxValue)"/>
							<xsl:value-of select="concat('&lt;','/owl:maxQualifiedCardinality&gt;')"/>
							<xsl:value-of select="concat('&lt;/','owl:Restriction','&gt;')"/>
						</xsl:when>
						<xsl:when test="$pMinValue=$pMaxValue">
							<xsl:value-of select="concat('&lt;','owl:Restriction','&gt;')"/>
							<xsl:value-of select="concat('&lt;owl:onProperty rdf:resource=&quot;',concat('#',$pProp),'&quot;/&gt;')"/>
							<xsl:value-of select="concat('&lt;','owl:onDataRange rdf:resource=&quot;',$pFiller,'&quot;/&gt;')"/>
							<xsl:value-of select="concat('&lt;','owl:qualifiedCardinality rdf:datatype=&quot;&amp;xsd;nonNegativeInteger&quot;&gt;')"/>
							<xsl:value-of select="string($pMinValue)"/>
							<xsl:value-of select="concat('&lt;','/owl:qualifiedCardinality&gt;')"/>
							<xsl:value-of select="concat('&lt;/','owl:Restriction','&gt;')"/>
						</xsl:when>
						<!-- if the minimum value is lower than the maximum value -->
						<xsl:when test="$pMinValue &lt; $pMaxValue and $pMinValue = '0'">
							<xsl:value-of select="concat('&lt;','owl:Class','&gt;')"/>
						    <xsl:value-of select="concat('&lt;','owl:intersectionOf rdf:parseType=&quot;Collection&quot;','&gt;')"/>
						    <xsl:value-of select="fkt:existentialDPRestriction($pProp,$pFiller,$pMinValue,'')"/>
						    <xsl:value-of select="fkt:existentialDPRestriction($pProp,$pFiller,'',$pMaxValue)"/>
							<xsl:value-of select="concat('&lt;/','owl:intersectionOf','&gt;')"/>
							<xsl:value-of select="concat('&lt;/','owl:Class','&gt;')"/>
						</xsl:when>
						<xsl:when test="$pMinValue &lt; $pMaxValue and $pMinValue != '0'">
							<xsl:value-of select="concat('&lt;','owl:Class','&gt;')"/>
						    <xsl:value-of select="concat('&lt;','owl:intersectionOf rdf:parseType=&quot;Collection&quot;','&gt;')"/>
						    <xsl:value-of select="fkt:existentialDPRestriction($pProp,$pFiller,$pMinValue,'')"/>
						    <xsl:value-of select="fkt:existentialDPRestriction($pProp,$pFiller,'',$pMaxValue)"/>
							<xsl:value-of select="concat('&lt;/','owl:intersectionOf','&gt;')"/>
							<xsl:value-of select="concat('&lt;/','owl:Class','&gt;')"/>
						</xsl:when>
					</xsl:choose>
				</xsl:when>
				<xsl:when test="($pMinValue!='' or $pMaxValue!='') and $pFiller=''">
					<xsl:value-of select="concat('&lt;','owl:Restriction','&gt;')"/>
					<xsl:value-of select="concat('&lt;owl:onProperty rdf:resource=&quot;',concat('#',$pProp),'&quot;/&gt;')"/>
					<xsl:value-of select="fkt:existentialHelper($pProp,$pMinValue,$pMaxValue)"/>
					<xsl:value-of select="concat('&lt;/','owl:Restriction','&gt;')"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="concat('&lt;','owl:Restriction','&gt;')"/>
					<xsl:value-of select="concat('&lt;owl:onProperty rdf:resource=&quot;',concat('#',$pProp),'&quot;/&gt;')"/>
					<xsl:value-of select="concat('&lt;owl:someValuesFrom rdf:resource=&quot;',$pFiller,'&quot;/&gt;')"/>
					<xsl:value-of select="concat('&lt;/','owl:Restriction','&gt;')"/>
				</xsl:otherwise>
			</xsl:choose>
</xsl:function>

<xsl:function name="fkt:existentialOPRestriction">
	<xsl:param name="pProp"/>
	<xsl:param name="pFiller"/>
	<xsl:param name="pMinValue"/>
	<xsl:param name="pMaxValue"/>
			<xsl:choose>
				<xsl:when test="($pMinValue!='' or $pMaxValue!='') and $pFiller!=''">
					<xsl:choose>
						<xsl:when test="$pMaxValue ='666'">
							<xsl:value-of select="concat('&lt;','owl:Restriction','&gt;')"/>
							<xsl:value-of select="concat('&lt;owl:onProperty rdf:resource=&quot;',concat('#',$pProp),'&quot;/&gt;')"/>
							<xsl:value-of select="concat('&lt;','owl:onClass rdf:resource=&quot;',$pFiller,'&quot;/&gt;')"/>
							<xsl:value-of select="concat('&lt;','owl:minQualifiedCardinality rdf:datatype=&quot;&amp;xsd;nonNegativeInteger&quot;&gt;')"/>
							<xsl:value-of select="$pMinValue"/>
							<xsl:value-of select="concat('&lt;','/owl:minQualifiedCardinality&gt;')"/>
							<xsl:value-of select="concat('&lt;/','owl:Restriction','&gt;')"/>
						</xsl:when>
						<xsl:when test="$pMinValue &gt;'0' and $pMaxValue=''">
							<xsl:value-of select="concat('&lt;','owl:Restriction','&gt;')"/>
							<xsl:value-of select="concat('&lt;owl:onProperty rdf:resource=&quot;',concat('#',$pProp),'&quot;/&gt;')"/>
							<xsl:value-of select="concat('&lt;','owl:onClass rdf:resource=&quot;',$pFiller,'&quot;/&gt;')"/>
							<xsl:value-of select="concat('&lt;','owl:minQualifiedCardinality rdf:datatype=&quot;&amp;xsd;nonNegativeInteger&quot;&gt;')"/>
							<xsl:value-of select="$pMinValue"/>
							<xsl:value-of select="concat('&lt;','/owl:minQualifiedCardinality&gt;')"/>
							<xsl:value-of select="concat('&lt;/','owl:Restriction','&gt;')"/>
						</xsl:when>		
						<xsl:when test="$pMinValue='' and $pMaxValue!=''">
							<xsl:value-of select="concat('&lt;','owl:Restriction','&gt;')"/>
							<xsl:value-of select="concat('&lt;owl:onProperty rdf:resource=&quot;',concat('#',$pProp),'&quot;/&gt;')"/>
							<xsl:value-of select="concat('&lt;','owl:onClass rdf:resource=&quot;',$pFiller,'&quot;/&gt;')"/>
							<xsl:value-of select="concat('&lt;','owl:maxQualifiedCardinality rdf:datatype=&quot;&amp;xsd;nonNegativeInteger&quot;&gt;')"/>
							<xsl:value-of select="string($pMaxValue)"/>
							<xsl:value-of select="concat('&lt;','/owl:maxQualifiedCardinality&gt;')"/>
							<xsl:value-of select="concat('&lt;/','owl:Restriction','&gt;')"/>
						</xsl:when>
						<xsl:when test="($pMinValue=$pMaxValue) and $pMaxValue!=''">
							<xsl:value-of select="concat('&lt;','owl:Restriction','&gt;')"/>
							<xsl:value-of select="concat('&lt;owl:onProperty rdf:resource=&quot;',concat('#',$pProp),'&quot;/&gt;')"/>
							<xsl:value-of select="concat('&lt;','owl:onClass rdf:resource=&quot;',$pFiller,'&quot;/&gt;')"/>
							<xsl:value-of select="concat('&lt;','owl:qualifiedCardinality rdf:datatype=&quot;&amp;xsd;nonNegativeInteger&quot;&gt;')"/>
							<xsl:value-of select="string($pMinValue)"/>
							<xsl:value-of select="concat('&lt;','/owl:qualifiedCardinality&gt;')"/>
							<xsl:value-of select="concat('&lt;/','owl:Restriction','&gt;')"/>
						</xsl:when>
						<!-- if the minimum value is lower than the maximum value -->
						<xsl:when test="$pMinValue &lt; $pMaxValue"> 		
						    <xsl:value-of select="concat('&lt;','owl:Class','&gt;')"/>
						    <xsl:value-of select="concat('&lt;','owl:intersectionOf rdf:parseType=&quot;Collection&quot;','&gt;')"/>
						    <xsl:value-of select="fkt:existentialOPRestriction($pProp,$pFiller,$pMinValue,'')"/>
						    <xsl:value-of select="fkt:existentialOPRestriction($pProp,$pFiller,'',$pMaxValue)"/>
							<xsl:value-of select="concat('&lt;/','owl:intersectionOf','&gt;')"/>
							<xsl:value-of select="concat('&lt;/','owl:Class','&gt;')"/>
						</xsl:when>
					</xsl:choose>
				</xsl:when>
				<xsl:when test="($pMinValue!='' or $pMaxValue!='') and $pFiller=''">
					<xsl:value-of select="concat('&lt;','owl:Restriction','&gt;')"/>
					<xsl:value-of select="concat('&lt;owl:onProperty rdf:resource=&quot;',concat('#',$pProp),'&quot;/&gt;')"/>
					<xsl:value-of select="fkt:existentialHelper($pProp,$pMinValue,$pMaxValue)"/>
					<xsl:value-of select="concat('&lt;/','owl:Restriction','&gt;')"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="concat('&lt;','owl:Restriction','&gt;')"/>
					<xsl:value-of select="concat('&lt;owl:onProperty rdf:resource=&quot;',concat('#',$pProp),'&quot;/&gt;')"/>
					<xsl:value-of select="concat('&lt;owl:someValuesFrom rdf:resource=&quot;',$pFiller,'&quot;/&gt;')"/>
					<xsl:value-of select="concat('&lt;/','owl:Restriction','&gt;')"/>
				</xsl:otherwise>
			</xsl:choose>
</xsl:function>

<xsl:function name="fkt:existentialHelper">
	<xsl:param name="pProp"/>
	<xsl:param name="pMinValue"/>
	<xsl:param name="pMaxValue"/>
		<xsl:choose>
			<xsl:when test="$pMinValue!='0'">
				<xsl:value-of select="concat('&lt;','owl:minCardinality rdf:datatype=&quot;&amp;xsd;nonNegativeInteger&quot;&gt;')"/>
				<xsl:value-of select="$pMinValue"/>
				<xsl:value-of select="concat('&lt;','/owl:minCardinality&gt;')"/>
			</xsl:when>
			<xsl:when test="$pMaxValue!=''">
				<xsl:value-of select="concat('&lt;','owl:maxCardinality rdf:datatype=&quot;&amp;xsd;nonNegativeInteger&quot;&gt;')"/>
				<xsl:value-of select="string($pMaxValue)"/>
				<xsl:value-of select="concat('&lt;','/owl:maxCardinality&gt;')"/>
			</xsl:when>
			<xsl:when test="$pMinValue=$pMaxValue">
				<xsl:value-of select="concat('&lt;','owl:Cardinality rdf:datatype=&quot;&amp;xsd;nonNegativeInteger&quot;&gt;')"/>
				<xsl:value-of select="string($pMinValue)"/>
				<xsl:value-of select="concat('&lt;','/owl:Cardinality&gt;')"/>
			</xsl:when>
		</xsl:choose>
</xsl:function>

<xsl:function name="fkt:cardinalityHelper">
	<xsl:param name="pProp"/>
	<xsl:param name="pMinMaxValue"/>
	<xsl:param name="pCardinalityValue" />
		<xsl:choose>
			<xsl:when test="$pMinMaxValue='min'">
				<xsl:value-of select="concat('&lt;','owl:minCardinality rdf:datatype=&quot;&amp;xsd;nonNegativeInteger&quot;&gt;')"/>
				<xsl:value-of select="$pCardinalityValue"/>
				<xsl:value-of select="concat('&lt;','/owl:minCardinality&gt;')"/>
			</xsl:when>
			<xsl:when test="$pMinMaxValue='max'">
				<xsl:value-of select="concat('&lt;','owl:maxCardinality rdf:datatype=&quot;&amp;xsd;nonNegativeInteger&quot;&gt;')"/>
				<xsl:value-of select="string($pCardinalityValue)"/>
				<xsl:value-of select="concat('&lt;','/owl:maxCardinality&gt;')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="concat('&lt;','owl:Cardinality rdf:datatype=&quot;&amp;xsd;nonNegativeInteger&quot;&gt;')"/>
				<xsl:value-of select="string($pCardinalityValue)"/>
				<xsl:value-of select="concat('&lt;','/owl:Cardinality&gt;')"/>
			</xsl:otherwise>
		</xsl:choose>
</xsl:function>

<!--  function to create existential restrictions inside a class definition for union of restrictions
	  param1: name of the restriction property
	  param2: name of the filler class/datatype-->
<xsl:function name="fkt:existentialUnionRestriction">
	<xsl:param name="pProp"/>
	<xsl:param name="pFiller"/>

</xsl:function>

<!-- recursive function to create owl list
	 param1:  list of owl:list elements whitespace separated
	 param2:  datatype of list elements -->
<xsl:function name="fkt:simpleList">
	<xsl:param name="pElems"/>
	<xsl:param name="pDType"/>
	<xsl:value-of select="concat('&lt;','rdf:List','&gt;')"/>
	<xsl:value-of select="concat('&lt;','rdf:first rdf:datatype=&quot;',$pDType,'&quot;&gt;')"/>
	<xsl:choose>
		<xsl:when test="$pElems=fkt:substring-after-last-match($pElems,' ')">
				<xsl:value-of select="fkt:substring-after-last-match($pElems,' ')"/>
				<xsl:value-of select="concat('&lt;/','rdf:first','&gt;')"/>
				<xsl:value-of select="string('&lt;rdf:rest rdf:resource=&quot;&amp;rdf;nil&quot;/&gt;')"/>
				<xsl:value-of select="string('&lt;/rdf:List&gt;')"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="substring-before($pElems,' ')"/>
			<xsl:value-of select="concat('&lt;/','rdf:first','&gt;')"/>
			<xsl:value-of select="string('&lt;rdf:rest&gt;')"/>
			<xsl:value-of select="fkt:simpleList(substring-after($pElems,' '),$pDType)"/>

			<xsl:value-of select="string('&lt;/rdf:rest&gt;')"/>
			<xsl:value-of select="string('&lt;/rdf:List&gt;')"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:function>


<!-- function to create datatype property with range enumeration list
	 param1: name
	 param2: range elements
	 param3: range elements datatype 
	 param4: formatted annotation property (optional) -->
<xsl:function name="fkt:dpListRange">
	<xsl:param name="pPropName"/>
	<xsl:param name="pElemsList"/>
	<xsl:param name="pDType"/>
	<xsl:param name="pAnn"/>
			<xsl:value-of select="concat('&lt;','owl:DatatypeProperty',' rdf:about=&quot;',$pPropName,'&quot;&gt;')"/>
			<!-- optional annotation -->
			<xsl:value-of select="$pAnn"/>
			<xsl:value-of select= "fkt:createAnnotation('comment',concat('Range values from following list expected: ',string-join($pElemsList,', ')))"/>
			<xsl:value-of select= "concat('&lt;rdfs:range rdf:resource=&quot;','&amp;xsd;string','&quot;/&gt;')"/>
			<xsl:value-of select="concat('&lt;','/owl:DatatypeProperty','&gt;&#10;')"/>	
					
</xsl:function>

<!-- function to create object/datatype properties
	 param1: property type (datatype/object)
	 param2: name
	 param3: type(optional) (e.g. functional property)
	 param4: domain (optional)
	 param5: range (optional) 
	 param6: annotation (optional)-->
<xsl:function name="fkt:createProperty">
	<xsl:param name="pPropType"/>
	<xsl:param name="pName"/>
	<xsl:param name="pType"/>
	<xsl:param name="pDomain"/>
	<xsl:param name="pRange"/>
	<xsl:param name="pAnn"/>
	
	<xsl:if test="$pName!=''">
		<xsl:value-of select="concat('&lt;owl:',$pPropType,'Property rdf:about=&quot;',$pName,'&quot;&gt;')"/>
		
		<!-- optional annotation -->
		<xsl:value-of select="$pAnn"/>
		
		<!-- optional type -->
		<xsl:if test="$pType!=''">
			<xsl:variable name="pTypes" select="tokenize($pType,'\s')"/>
			<xsl:for-each select="$pTypes">
				<xsl:value-of select= "concat('&lt;','rdf:type rdf:resource=&quot;',.,'&quot;/&gt;')"/>
			</xsl:for-each>
	    </xsl:if>
		<xsl:choose>
		<!-- make a union of the domain classes if there are more than one domain class -->
			<xsl:when test="count($pDomain)>1">
				<xsl:value-of select= "concat('&lt;rdfs:domain','&gt;')"/>
					<xsl:value-of select= "concat('&lt;owl:Class','&gt;')"/>
						<xsl:value-of select= "concat('&lt;owl:unionOf rdf:parseType=&quot;Collection&quot;','&gt;')"/>
							<xsl:for-each select="$pDomain">
								<!-- with concat # the qualified name of the resource is created and thus connected to the xml:base namespace -->
								<xsl:value-of select= "concat('&lt;rdf:Description rdf:about=&quot;',concat('#',.),'&quot;/&gt;')"/>
							</xsl:for-each>
						<xsl:value-of select= "concat('&lt;/owl:unionOf','&gt;')"/>
					<xsl:value-of select= "concat('&lt;/owl:Class','&gt;')"/>
				<xsl:value-of select= "concat('&lt;/rdfs:domain','&gt;')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:if test="$pDomain!=''">
					<xsl:value-of select= "concat('&lt;rdfs:domain rdf:resource=&quot;',$pDomain,'&quot;/&gt;')"/>
				</xsl:if>		
			</xsl:otherwise>
		</xsl:choose>
		<xsl:choose>
		<!-- make a union of the range classes if there are more than one range class -->
			<xsl:when test="count($pRange)>1">
				<xsl:value-of select= "concat('&lt;rdfs:range','&gt;')"/>
		<!-- differentiation if it is a datatype or object property -->		
				<xsl:choose>
					<xsl:when test="$pPropType='Object'">
						<xsl:value-of select= "concat('&lt;owl:Class','&gt;')"/>
					</xsl:when>
					<xsl:when test="$pPropType='Datatype'">
						<xsl:value-of select= "concat('&lt;rdfs:Datatype','&gt;')"/>
					</xsl:when>
				</xsl:choose>
					
						<xsl:value-of select= "concat('&lt;owl:unionOf rdf:parseType=&quot;Collection&quot;','&gt;')"/>
							<xsl:for-each select="$pRange">
								<!-- with concat # the qualified name of the resource is created and thus connected to the xml:base namespace -->
								<xsl:value-of select= "concat('&lt;rdf:Description rdf:about=&quot;',fkt:testOfType(.),'&quot;/&gt;')"/>
							</xsl:for-each>
						<xsl:value-of select= "concat('&lt;/owl:unionOf','&gt;')"/>
		<!-- differentiation if it is a datatype or object property -->	
				<xsl:choose>
					<xsl:when test="$pPropType='Object'">
						<xsl:value-of select= "concat('&lt;/owl:Class','&gt;')"/>
					</xsl:when>
					<xsl:when test="$pPropType='Datatype'">
						<xsl:value-of select= "concat('&lt;/rdfs:Datatype','&gt;')"/>
					</xsl:when>
				</xsl:choose>
				<xsl:value-of select= "concat('&lt;/rdfs:range','&gt;')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:if test="$pRange!=''">
					<xsl:value-of select= "concat('&lt;rdfs:range rdf:resource=&quot;',$pRange,'&quot;/&gt;')"/>
				</xsl:if>		
			</xsl:otherwise>
		</xsl:choose>
		<xsl:value-of select="concat('&lt;/owl:',$pPropType,'Property&gt;','')"/>
	</xsl:if>

</xsl:function>

</xsl:stylesheet>