<!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
		XSLT-Stylesheet responsible for transforming gbXML to RDF/OWL
		Author: Mario J. Kofler
		Date: November 2012
		Version: 0.2
		Description: 
		This part takes care of the concrete house model, means the Individuals of the ontology.
		Update November 2012: changed handling of element "DocumentHistory" and child elements because of change in gbXML versions 5.x with respect to "ProductName" element
	 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-->
<!DOCTYPE rdf:RDF [
    <!ENTITY owl "http://www.w3.org/2002/07/owl#" >
    <!ENTITY xsd "http://www.w3.org/2001/XMLSchema#" >
    <!ENTITY owl2xml "http://www.w3.org/2006/12/owl2-xml#" >
    <!ENTITY rdfs "http://www.w3.org/2000/01/rdf-schema#" >
    <!ENTITY rdf "http://www.w3.org/1999/02/22-rdf-syntax-ns#" >
    <!ENTITY gbOnt "http://www.auto.tuwien.ac.at/downloads/thinkhome/ontology/BuildingOntology.owl#" >
]>
<xsl:stylesheet version="2.0" 
				xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
				xmlns:gbx="http://www.gbxml.org/schema"
				xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"
				xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
				xmlns:owl="http://www.w3.org/2002/07/owl#"
				xmlns:fkt="http://www.auto.tuwien.ac.at/mkofler/functions"
				>

	<xsl:template match="gbx:gbXML" mode="complete">
		
		<xsl:variable name="rootElem" select="current()/local-name()"/>
		
		<!-- elements that are not needed in the generated OWL building model because they are just needed for CAD 3D graphics -->
		<xsl:variable name="ignoreElementNodes" select ="('PolyLoop','PlanarGeometry','CartesianPoint','Coordinate','ShellGeometry','ClosedShell')"/>

<!-- create surface individuals -->
<xsl:apply-templates select="//gbx:PolyLoop" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<xsl:apply-templates select="//gbx:PlanarGeometry" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<xsl:apply-templates select="//gbx:RectangularGeometry" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<xsl:apply-templates select="//gbx:CartesianPoint" mode="individual">		
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<xsl:apply-templates select="//gbx:Opening" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<xsl:apply-templates select="//gbx:Surface" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<!--  create space individuals -->
<xsl:apply-templates select="//gbx:ClosedShell" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<xsl:apply-templates select="//gbx:ShellGeometry" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<xsl:apply-templates select="//gbx:InfiltrationFlow" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<!-- create spaces and space boundaries -->
<xsl:apply-templates select="//gbx:Space" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<xsl:apply-templates select="//gbx:SpaceBoundary" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<!-- create building individual -->
<xsl:apply-templates select="//gbx:Building" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<!-- create campus individual -->
<xsl:apply-templates select="//gbx:Location" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<xsl:apply-templates select="gbx:Campus" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<!-- create construction individuals -->
<xsl:apply-templates select="//gbx:Cost" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<xsl:apply-templates select="gbx:Construction" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<!-- create other important individuals -->

<xsl:apply-templates select="gbx:Layer" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<xsl:apply-templates select="gbx:Material" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<xsl:apply-templates select="gbx:WindowType" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<xsl:apply-templates select="//gbx:PersonInfo" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<xsl:apply-templates select="//gbx:ProgramInfo" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<xsl:apply-templates select="gbx:DocumentHistory" mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>

<!-- create gbXML individual -->
<xsl:apply-templates select="." mode="individual">
    <xsl:with-param  name="pIgnoreElems" select="$ignoreElementNodes"/>
</xsl:apply-templates>


	</xsl:template>	
	
	<!--  template for elements with deeper hierarchies and child object-properties -->
	<xsl:template match="	 gbx:ClosedShell
							|gbx:ShellGeometry
							|gbx:PolyLoop
							|gbx:PlanarGeometry
							|gbx:RectangularGeometry
							|gbx:Construction
							|gbx:Opening
							|gbx:Surface
							|gbx:Space
							|gbx:SpaceBoundary 
							|gbx:Building
							|gbx:Campus
							|gbx:gbXML
							|gbx:DocumentHistory
							" mode="individual">
	<xsl:param name="pIgnoreElems"/>
				
		<xsl:variable name="rootElem" select="current()/local-name()"/>
		<!-- test for elements to ignore (coordinates and similar elements which are used for graphical representation in CAD -->
		<xsl:if test="not(fkt:isIgnoreNode($rootElem,$pIgnoreElems)) ">
			<!-- create individuals -->		
			<xsl:for-each select="child::*">
			<!-- create qualified name of descendant -->
			<xsl:variable name="qualifiedName" select="concat('#',current()/local-name())"/>
			<xsl:choose>
				<xsl:when test="document('../../OWL/outXSD.owl')//owl:Class[@rdf:about=$qualifiedName] and (count(child::*)=0)">
					<xsl:variable name="indvName" select="concat(current()/local-name(),'_ID=',generate-id())"/>	
					<xsl:variable	name="childDPHelper" select="   if (normalize-space(.)!='')
																	then (if (string-join(fkt:createDPInd(current()/local-name(),current()/descendant::*/local-name(),descendant::*),' ')='') 
					   											    	  then (string-join(fkt:createDPInd(current()/local-name(),'NativeValue',.),' '))
					   											    	  else(string-join(fkt:createDPInd(current()/local-name(),current()/descendant::*/local-name(),descendant::*),' ')))
					   											   	else('')"/> 
					<xsl:value-of 	disable-output-escaping="yes"
									select="fkt:createIndividual(	concat(current()/local-name(),'_ID=',generate-id()),current()/local-name(),'',
																	concat(string-join(fkt:createDPInd(current()/local-name(),current()/@*/local-name(),current()/@*),' '),$childDPHelper))"/>
				</xsl:when>
			</xsl:choose>
			</xsl:for-each>	
			
			<!-- create connection to main individual -->
			<xsl:choose>
			<xsl:when test="@id">
				<xsl:value-of 	disable-output-escaping="yes"
								select="concat('&lt;','owl:Thing',' rdf:about=&quot;',concat('#',$rootElem,'_ID=',@id),'&quot;&gt;')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of 	disable-output-escaping="yes"
								select="concat('&lt;','owl:Thing',' rdf:about=&quot;',concat('#',$rootElem,'_ID=',generate-id()),'&quot;&gt;')"/>
			</xsl:otherwise>
			</xsl:choose>
			
			<xsl:if test="gbx:Name">
				<xsl:value-of	disable-output-escaping="yes" 
								select="fkt:createAnnotation('comment',concat('Name of ', $rootElem,': ',gbx:Name))"/>
			</xsl:if>
					
			<xsl:for-each select="child::*">
				<!-- create Object Properties -->	
				<!-- create qualified name of child -->
				<xsl:variable name="qualifiedName" select="concat('#','contains',current()/local-name())"/>
				<xsl:choose>
				<xsl:when test="document('../../OWL/outXSD.owl')//owl:ObjectProperty[@rdf:about=$qualifiedName]">
				
					<!-- test for elements to ignore (coordinates and similar elements which are used for graphical representation in CAD -->
					<xsl:if test="not(fkt:isIgnoreNode(current()/local-name(),$pIgnoreElems)) ">
						<xsl:choose>
						<xsl:when test="@id">
							<xsl:value-of 	disable-output-escaping="yes"
											select="fkt:createOPInd(current()/local-name(),concat('#',current()/local-name(),'_ID=',@id))"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of 	disable-output-escaping="yes"
											select="fkt:createOPInd(current()/local-name(),concat('#',current()/local-name(),'_ID=',generate-id()))"/>
						</xsl:otherwise>
						</xsl:choose>
					</xsl:if>
					
				</xsl:when>
				<xsl:otherwise>
				<!-- create Datatype Properties (Sub-Elements) -->
					<xsl:value-of  	disable-output-escaping="yes"
									select="fkt:createDPInd(current()/parent::*/local-name(),current()/local-name(),current())"/>
				</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>
				<!-- create Datatype Properties (Attributes) -->
				<xsl:value-of 	disable-output-escaping="yes"
								select="fkt:createDPInd(current()/local-name(),current()/@*/local-name(),current()/@*)"/>
			<xsl:value-of 	disable-output-escaping="yes"
							select="concat('&lt;/','owl:Thing','&gt;')"/>
		</xsl:if>					
	</xsl:template>
				
	<!-- template for simple elements which only contain datatype properties or simple object properties without deeper hierarchies -->
	<xsl:template match="	 gbx:CartesianPoint
							|gbx:InfiltrationFlow
							|gbx:Location
						 	|gbx:Layer
						 	|gbx:Material
						 	|gbx:Cost
						 	|gbx:WindowType
						 	|gbx:PersonInfo
						 	|gbx:ProgramInfo" mode="individual">
	<xsl:param name="pIgnoreElems"/>
		
		
		<xsl:variable name="rootElem" select="current()/local-name()"/>
		<!-- test for elements to ignore (coordinates and similar elements which are used for graphical representation in CAD)-->
		<xsl:if test="not(fkt:isIgnoreNode($rootElem,$pIgnoreElems)) ">
			<!-- create individuals -->		
			<xsl:for-each select="descendant::*">
			<!-- create qualified name of descendant -->
			<xsl:variable name="qualifiedName" select="concat('#',current()/local-name())"/>
			<xsl:choose>
				<!-- test if the actual descendant is a class in the owl schema and then create the individual with datatype properties -->
				<xsl:when test="document('../../OWL/outXSD.owl')//owl:Class[@rdf:about=$qualifiedName]">
					<!-- create new datatype properties -->
						<xsl:variable	name="childDPHelper" select="   if (normalize-space(.)!='')
																		then (if (string-join(fkt:createDPInd(current()/local-name(),current()/descendant::*/local-name(),descendant::*),' ')='') 
						   													  then (string-join(fkt:createDPInd(current()/local-name(),'NativeValue',.),' '))
																			  else(string-join(fkt:createDPInd(current()/local-name(),current()/descendant::*/local-name(),descendant::*),' ')))
																		else ('')"/>  

						<!-- handling of empty elements, which of course should not be added to the ontology -->
						<xsl:choose>
						<xsl:when test="@id"> 											    
							<xsl:value-of 	disable-output-escaping="yes"
											select="fkt:createIndividual(	concat(current()/local-name(),'_ID=',@id),current()/local-name(),'',
																			concat(string-join(fkt:createDPInd(current()/local-name(),current()/@*/local-name(),current()/@*),' '),$childDPHelper))"/>
						</xsl:when>
						<xsl:otherwise>  											    
							<xsl:value-of 	disable-output-escaping="yes"
											select="fkt:createIndividual(	concat(current()/local-name(),'_ID=',generate-id()),current()/local-name(),'',
																			concat(string-join(fkt:createDPInd(current()/local-name(),current()/@*/local-name(),current()/@*),' '),$childDPHelper))"/>
						</xsl:otherwise>
						</xsl:choose>
		
				</xsl:when>
			</xsl:choose>
			</xsl:for-each>	
			
			<!-- create connection to main individual -->
			<xsl:choose>
			<xsl:when test="@id">
				<xsl:value-of 	disable-output-escaping="yes"
								select="concat('&lt;','owl:Thing',' rdf:about=&quot;',concat('#',$rootElem,'_ID=',@id),'&quot;&gt;')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of 	disable-output-escaping="yes"
								select="concat('&lt;','owl:Thing',' rdf:about=&quot;',concat('#',$rootElem,'_ID=',generate-id()),'&quot;&gt;')"/>
			</xsl:otherwise>
			</xsl:choose>
			
			<xsl:if test="gbx:Name">
				<xsl:value-of	disable-output-escaping="yes" 
								select="fkt:createAnnotation('comment',concat('Name of ', $rootElem,': ',gbx:Name))"/>
			</xsl:if>
					
			<xsl:for-each select="child::*">
				<!-- create Object Properties -->
				<!-- create qualified name of child -->
				<xsl:variable name="qualifiedName" select="concat('#','contains',current()/local-name())"/>	
				<xsl:choose>
				<xsl:when test="document('../../OWL/outXSD.owl')//owl:ObjectProperty[@rdf:about=($qualifiedName)]">
		
					<!-- test for elements to ignore (coordinates and similar elements which are used for graphical representation in CAD) -->
					<xsl:if test="not(fkt:isIgnoreNode(current()/local-name(),$pIgnoreElems))">
						<xsl:choose>
						<xsl:when test="@id">
							<xsl:value-of 	disable-output-escaping="yes"
											select="fkt:createOPInd(current()/local-name(),concat('#',current()/local-name(),'_ID=',@id))"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of 	disable-output-escaping="yes"
											select="fkt:createOPInd(current()/local-name(),concat('#',current()/local-name(),'_ID=',generate-id()))"/>
						</xsl:otherwise>
						</xsl:choose>
					</xsl:if>
					
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of  	disable-output-escaping="yes"
									select="fkt:createDPInd(current()/parent::*/local-name(),current()/local-name(),current())"/>
				</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>
				<xsl:value-of 	disable-output-escaping="yes"
								select="fkt:createDPInd(current()/local-name(),current()/@*/local-name(),current()/@*)"/>
			<xsl:value-of 	disable-output-escaping="yes"
							select="concat('&lt;/','owl:Thing','&gt;')"/>
		
		</xsl:if>	
	</xsl:template>
</xsl:stylesheet>