<!-- +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
		XSLT-Stylesheet responsible for transforming gbXML to RDF/OWL
		Author: Mario J. Kofler
		Date: November 2012
		Version: 0.2
		Description: 
		Transfers the gbXML Schema to RDF/OWL. Takes Approach independent
		of actual instatiations of gbXML elements.
		This part is responsible for the creation of OWL properties
	 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-->
<!DOCTYPE rdf:RDF [
    <!ENTITY owl "http://www.w3.org/2002/07/owl#" >
    <!ENTITY xsd "http://www.w3.org/2001/XMLSchema#" >
    <!ENTITY owl2xml "http://www.w3.org/2006/12/owl2-xml#" >
    <!ENTITY rdfs "http://www.w3.org/2000/01/rdf-schema#" >
    <!ENTITY rdf "http://www.w3.org/1999/02/22-rdf-syntax-ns#" >
    <!ENTITY gbOnt "http://www.auto.tuwien.ac.at/downloads/thinkhome/ontology/BuildingOntology.owl#" >
    <!ENTITY gbdt "http://www.gbxml.org/schema/0-37/GreenBuildingXML.xsd#" >
]>

<xsl:stylesheet version="2.0" 
				xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
				xmlns:xsd="http://www.w3.org/2001/XMLSchema"
				xmlns:owl="http://www.w3.org/2002/07/owl#"
				xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
				xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"
				xmlns:fkt="http://www.auto.tuwien.ac.at/mkofler/functions"
				xmlns:gbdt="http://www.gbxml.org/schema/0-37/GreenBuildingXML.xsd#"
				exclude-result-prefixes="fkt gbdt"
				>		

	<!-- template for creating class datatype properties (hasValue, datatype properties)
		 attributes in the XSD file are generally realized as datatype properties -->	
	<xsl:template name="createDatatypeProperties">
	
	<!-- UPDATE 26.11.2012: fetch the global variable from masterXSD.xsl file. -->
	<xsl:variable name="pgbXMLVersion" select="$vgbXMLVersion"/>	
	
	<!-- grouping, to find out multiple datatype declarations (hence, multiple attribute declarations with same attribute name) -->
		<xsl:for-each-group select="//xsd:attribute" group-by="@name">
				<xsl:choose>
				<xsl:when test="count(current-group())>1">

					<!-- CHANGE 19.06.2010: more than one xsd:datatype: create restriction with union of all values -->
					<xsl:value-of 	disable-output-escaping="yes"
									select="fkt:createProperty('Datatype',concat('#',fkt:createDPName(@name)),'','',current-group()/@type,fkt:createAnnotation('comment',xsd:annotation/xsd:documentation))"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:choose>
						<xsl:when test="@type!=''">
							<!-- UPDATE 26.11.2012: The document to compare is the current gbXML schema file, which is located in a subdirectory of XSD denoting the version (e.g. 0.37) and stored in the variable pgbXMLVersion -->
							<xsl:variable name="elem_exists" select="document(concat('../../XSD/',$pgbXMLVersion,'/GreenBuildingXML.xsd'))/xsd:schema/xsd:element[@name=fkt:capitalize-first(current-grouping-key())]"/>
							<xsl:choose>
							<xsl:when test="not($elem_exists)">
								
								<xsl:value-of 	disable-output-escaping="yes"
												select="fkt:createProperty('Datatype',concat('#',fkt:createDPName(@name)),'','',fkt:testOfType(@type),fkt:createAnnotation('comment',xsd:annotation/xsd:documentation))"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of 	disable-output-escaping="yes"
												select="fkt:createProperty('Datatype',concat('#',fkt:createDPName(@name)),'','',fkt:testOfType(@type),fkt:createAnnotation('comment',xsd:annotation/xsd:documentation))"/>
							</xsl:otherwise>
							</xsl:choose>
						</xsl:when>
						<xsl:otherwise>
							<xsl:choose>
								<xsl:when test="xsd:simpleType/xsd:union">
									<xsl:value-of disable-output-escaping="yes"
												  select="fkt:dpListRange(concat('#',fkt:createDPName(@name)),'','',fkt:createAnnotation('comment',xsd:annotation/xsd:documentation))"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of disable-output-escaping="yes"
												  select="fkt:dpListRange(concat('#',fkt:createDPName(@name)),xsd:simpleType/xsd:restriction/xsd:enumeration/@value,fkt:testOfType('xsd:string'),fkt:createAnnotation('comment',xsd:annotation/xsd:documentation))"/>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:otherwise>
				</xsl:choose>
		</xsl:for-each-group>
	</xsl:template>

<!-- template for creating complex object properties
	 object properties domain is a unionOf more than one class. -->	
<xsl:template name="createObjectProperties">
	<xsl:variable name="datatypeElements" select="''"/>
	
	<xsl:value-of select="$datatypeElements"/>
	
	<xsl:for-each-group select="xsd:element/xsd:complexType/xsd:choice/xsd:element" group-by="@ref">
	<xsl:if test="not(@ref = 'UtilityRate' or
	  				  @ref = 'aecXML' or
	  				  @ref = 'GeneralGeometry')">
	<!-- relatively complex action for taking care of already defined datatype properties which are elements with 0 or 1 children-->
	<xsl:for-each select="../../../../xsd:element">
		<xsl:choose>
			<xsl:when test="@name=current-grouping-key()">
				<xsl:choose>
					<xsl:when test="count(child::*)=0">
						<!-- do nothing -->
					</xsl:when>	
					<xsl:when test="count(child::*)>=1 and not(xsd:simpleType/xsd:restriction)">
						<xsl:choose>
							<!-- test simpleType then it should be an ordinary datatype property and no object property. -->
							<xsl:when test="(count(child::*)=1) and (child::*/local-name()='simpleType')">
								<!-- do nothing -->
							</xsl:when>
							<!-- test annotation then it should be an ordinary datatype property and no object property. -->
							<xsl:when test="(count(child::*)=1) and (child::*/local-name()='annotation')">
								<!-- do nothing -->
							</xsl:when>
							<xsl:otherwise>
								<!-- create new object property -->
								<xsl:element name="owl:ObjectProperty">
								<xsl:attribute name="rdf:about">
									<xsl:value-of select="concat('#','contains',fkt:capitalize-first(current-grouping-key()))"/>
								</xsl:attribute>
									<xsl:choose>
										<!--  make a union of the domain classes if there are more than one domain class -->
										<xsl:when test="count(current-group())>1">
											<xsl:element name="rdfs:domain">
												<xsl:element name="owl:Class">
													<xsl:element name="owl:unionOf">
													<xsl:attribute name="rdf:parseType">
														<xsl:value-of select="'Collection'"/>
													</xsl:attribute>
														<xsl:for-each select="current-group()">
															<xsl:element name="rdf:Description">
															<xsl:attribute name="rdf:about">
																<!-- with concat # the qualified name of the resource is created and thus connected to the xml:base namespace -->
																<xsl:value-of select="concat('#',../../../@name)"/>
															</xsl:attribute>
															</xsl:element>
														</xsl:for-each>
													</xsl:element>
												</xsl:element>
											</xsl:element>
										</xsl:when>
										<xsl:otherwise>
											<xsl:element name="rdfs:domain">
											<xsl:attribute name="rdf:resource">
												<!-- create qualified name with the help of # in order to link the resource to the xml:base namespace -->
												<xsl:value-of select="concat('#',current-group()/../../../@name)"/>
											</xsl:attribute>
											</xsl:element>			
										</xsl:otherwise>
									</xsl:choose>
									<xsl:element name="rdfs:range">
										<xsl:attribute name="rdf:resource">
											<!-- create qualified name with the help of # in order to link the resource to the xml:base namespace -->
											<xsl:value-of select="concat('#',current-grouping-key())"/>
										</xsl:attribute>
									</xsl:element>
								</xsl:element>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
				</xsl:choose>
			</xsl:when>
		</xsl:choose>
	</xsl:for-each>
	</xsl:if>
	</xsl:for-each-group>
</xsl:template>

</xsl:stylesheet>