#!/bin/sh
if [ "$#" -ne "6" ] 
then
	echo "Usage: latex.sh <output_file> <xsd_schema_input> <xml_input> <xslt_xsd_master> <xslt_xml_master> <gbxml_version>
		     Parameters:
				1. output_file: 	name of the output file to write to (eg. building.owl)
				2. xsd_schema_input: 	schema file to create the ontology model from (eg. gbXML.xsd)
				3. xml_input:		actual individual house in gbXML format, that should be transformed to OWL (e.g. building.xml)
				4. xslt_xsd_master	XSL Transformation sheet to translate the gbXML.xsd Schema into OWL
				5. xslt_xml_master	XSL Transformation sheet to translate the gbXML.xml Model into OWL
				6. gbxml_version	Version of the gbXML Schema to be used (eg. 0.37)"
else
	#save the directory for further usage
	HOMEDIR=`pwd`
	## creation step ##
	#create building model in owl
        `java -cp /opt/saxonb/saxon9.jar net.sf.saxon.Transform $2 $4 vgbXMLVersion=$6 > ../OWL/outXSD.owl`
	#create individual building in owl
	`java -cp /opt/saxonb/saxon9.jar net.sf.saxon.Transform $3 $5 vgbXMLVersion=$6 > ../OWL/outXML.owl`

	## concatenation step ##
	#strip last line of building model for concatenation
        `sed '$d' ../OWL/outXSD.owl > $1`
	#strip first few lines and concatenate to gbBuilding  concatenation
	`sed '1,6d' ../OWL/outXML.owl >> $1`

	## remove auxiliary files step ##
	#remove temporary helper files
	`rm ../OWL/outXSD.owl ../OWL/outXML.owl`

fi
